/*
 * Copyright (c) 2000-2009 TeamDev Ltd. All rights reserved.
 * TeamDev PROPRIETARY and CONFIDENTIAL.
 * Use is subject to license terms.
 */

package com.teamdev.xpcom.mozilla.demo.components.icon;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;
import java.util.*;
import java.awt.image.BufferedImage;
import java.awt.*;
import java.net.URL;
import java.io.InputStream;
import java.io.IOException;

/**
 * This class lets you read image files
 * in the formats that the JDK supports from url.
 * Multi-file images are supported.
 */
public class JavaImage {

    private static Set<String> imageFormats = getReaderFileSuffixes();

    private String imageUrl = "";

    private String imageType = "";

    public JavaImage(String urlToImage) throws IOException {
        imageUrl = urlToImage;
        imageType = getImageType(imageUrl);
    }

    public Image getImage() throws IOException {
        Image[] images = getImages();
        if (images != null && images.length > 0) {
            return images[0];
        } else {
            return null;
        }
    }

    public static boolean isJavaSupportImage(String url) throws IOException {
        String imageType = getPartOfImageType(url);
        for (String format : imageFormats) {
            if (format.contains(imageType) || imageType.contains(format)) {
                return true;
            }
        }
        return false;
    }

    public Image getImage(int prefferedWidth, int prefferedHight) throws IOException {
        Image[] images = getImages();
        if (images != null && images.length > 0) {
            for (Image image : images) {
                int currentWidth = ((BufferedImage) image).getWidth();
                int currentHeight = ((BufferedImage) image).getHeight();
                if (currentWidth == prefferedWidth && currentHeight == prefferedHight) {
                    return image;
                }
            }
            return images[0];
        } else {
            return null;
        }
    }

    private Image[] getImages() throws IOException {
        URL urlImage = new URL(imageUrl);
        InputStream inputStream = urlImage.openStream();

        Iterator<ImageReader> iter = ImageIO.getImageReadersBySuffix(imageType);
        ImageReader reader = iter.next();
        ImageInputStream imageIn = ImageIO.createImageInputStream(inputStream);
        reader.setInput(imageIn);
        int count = reader.getNumImages(true);
        Image[] images = new BufferedImage[count];
        for (int i = 0; i < count; i++) {
            images[i] = reader.read(i);
        }
        return images;
    }

    private static String getImageType(String urlToImage) throws IOException {
        String partOfImageType = getPartOfImageType(urlToImage);
        for (String imageFormat : imageFormats) {
            if (partOfImageType.contains(imageFormat) || imageFormat.contains(partOfImageType)) {
                return imageFormat;
            }
        }
        return null;
    }

    private static String getPartOfImageType(String urlToImage) throws IOException {
        URL currentURL = new URL(urlToImage);
        InputStream inputStream = currentURL.openStream();
        byte[] imageInfo = new byte[8];
        inputStream.read(imageInfo, 0, imageInfo.length);
        return  new String(imageInfo, "UTF-8").toLowerCase();
    }

    private static Set<String> getReaderFileSuffixes() {
        TreeSet<String> readerSuffixes = new TreeSet<String>();
        for (String name : ImageIO.getReaderFormatNames()) {
            Iterator<ImageReader> iter = ImageIO.getImageReadersByFormatName(name);
            while (iter.hasNext()) {
                ImageReader reader = iter.next();
                String[] s = reader.getOriginatingProvider().getFileSuffixes();
                readerSuffixes.addAll(Arrays.asList(s));
            }
        }
        return readerSuffixes;
    }

    public static void main(String[] args) {
        try {
            String url = "http://Pepsi.com/favicon.ico";
            if (JavaImage.isJavaSupportImage(url)) {
                JavaImage javaImage = new JavaImage(url);
                BufferedImage bufferedImage = (BufferedImage) javaImage.getImage(16, 16);
                System.out.println("bufferedImage.getHeight() = " + bufferedImage.getHeight());
            } else {
                System.out.println("Error");
                System.exit(-1);
            }

            String s = getPartOfImageType("http://Pepsi.com/favicon.ico");
            System.out.println("String = " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
